﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;
using System.Security.Cryptography;
using System.ServiceModel;
using HeuristicLab.Services.OKB.DataAccess;

namespace HeuristicLab.Services.OKB {
  /// <summary>
  /// Implementation of the OKB administration service (interface <see cref="IAdministrationService"/>).
  /// </summary>
  [ServiceBehavior(IncludeExceptionDetailInFaults = true)]
  public class AdministrationService : IAdministrationService {
    #region Platform Methods
    public DataTransfer.Platform GetPlatform(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.Platforms.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Platform> GetPlatforms() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.Platforms.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddPlatform(DataTransfer.Platform dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Platform entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.Platforms.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdatePlatform(DataTransfer.Platform dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Platform entity = okb.Platforms.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeletePlatform(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Platform entity = okb.Platforms.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Platforms.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region DataType Methods
    public DataTransfer.DataType GetDataType(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.DataTypes.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.DataType> GetDataTypes() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.DataTypes.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddDataType(DataTransfer.DataType dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.DataType entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.DataTypes.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateDataType(DataTransfer.DataType dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.DataType entity = okb.DataTypes.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteDataType(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.DataType entity = okb.DataTypes.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.DataTypes.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region BinaryData Methods
    public long GetBinaryDataId(byte[] hash) {
      using (OKBDataContext okb = new OKBDataContext()) {
        var id = okb.BinaryDatas.Where(x => x.Hash.Equals(hash)).Select(x => x.Id).FirstOrDefault();
        if (id == 0) return -1;
        else return id;
      }
    }
    public long AddBinaryData(DataTransfer.BinaryData dto) {
      byte[] hash;
      using (SHA1 sha1 = SHA1.Create()) {
        hash = sha1.ComputeHash(dto.Data);
      }

      using (OKBDataContext okb = new OKBDataContext()) {
        var id = okb.BinaryDatas.Where(x => x.Hash.Equals(hash)).Select(x => x.Id).FirstOrDefault();
        if (id != 0) {
          return id;
        } else {
          DataAccess.BinaryData entity = Convert.ToEntity(dto); entity.Id = 0;
          okb.BinaryDatas.InsertOnSubmit(entity);
          okb.SubmitChanges();
          return entity.Id;
        }
      }
    }
    public void DeleteBinaryData(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.BinaryData entity = okb.BinaryDatas.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.BinaryDatas.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region AlgorithmClass Methods
    public DataTransfer.AlgorithmClass GetAlgorithmClass(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.AlgorithmClasses.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.AlgorithmClass> GetAlgorithmClasses() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.AlgorithmClasses.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddAlgorithmClass(DataTransfer.AlgorithmClass dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.AlgorithmClass entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.AlgorithmClasses.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateAlgorithmClass(DataTransfer.AlgorithmClass dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.AlgorithmClass entity = okb.AlgorithmClasses.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteAlgorithmClass(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.AlgorithmClass entity = okb.AlgorithmClasses.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.AlgorithmClasses.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region Algorithm Methods
    public DataTransfer.Algorithm GetAlgorithm(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.Algorithms.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Algorithm> GetAlgorithms() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.Algorithms.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddAlgorithm(DataTransfer.Algorithm dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Algorithm entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.Algorithms.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateAlgorithm(DataTransfer.Algorithm dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Algorithm entity = okb.Algorithms.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteAlgorithm(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Algorithm entity = okb.Algorithms.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Algorithms.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    public IEnumerable<Guid> GetAlgorithmUsers(long algorithmId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.AlgorithmUsers.Where(x => x.AlgorithmId == algorithmId).Select(x => x.UserId).ToArray();
      }
    }
    public void UpdateAlgorithmUsers(long algorithmId, IEnumerable<Guid> users) {
      using (OKBDataContext okb = new OKBDataContext()) {
        okb.AlgorithmUsers.DeleteAllOnSubmit(okb.AlgorithmUsers.Where(x => x.AlgorithmId == algorithmId));
        okb.AlgorithmUsers.InsertAllOnSubmit(users.Select(x => new DataAccess.AlgorithmUser { AlgorithmId = algorithmId, UserId = x }));
        okb.SubmitChanges();
      }
    }
    public DataTransfer.BinaryData GetAlgorithmData(long algorithmId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Algorithm>(x => x.BinaryData);
        okb.LoadOptions = dlo;
        return Convert.ToDto(okb.Algorithms.Where(x => x.Id == algorithmId).Select(x => x.BinaryData).FirstOrDefault());
      }
    }
    #endregion

    #region ProblemClass Methods
    public DataTransfer.ProblemClass GetProblemClass(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.ProblemClasses.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.ProblemClass> GetProblemClasses() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.ProblemClasses.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddProblemClass(DataTransfer.ProblemClass dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemClass entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.ProblemClasses.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateProblemClass(DataTransfer.ProblemClass dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemClass entity = okb.ProblemClasses.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteProblemClass(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemClass entity = okb.ProblemClasses.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.ProblemClasses.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region Problem Methods
    public DataTransfer.Problem GetProblem(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.Problems.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Problem> GetProblems() {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.Problems.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddProblem(DataTransfer.Problem dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Problem entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.Problems.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateProblem(DataTransfer.Problem dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Problem entity = okb.Problems.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteProblem(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Problem entity = okb.Problems.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Problems.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    public IEnumerable<Guid> GetProblemUsers(long problemId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.ProblemUsers.Where(x => x.ProblemId == problemId).Select(x => x.UserId).ToArray();
      }
    }
    public void UpdateProblemUsers(long problemId, IEnumerable<Guid> users) {
      using (OKBDataContext okb = new OKBDataContext()) {
        okb.ProblemUsers.DeleteAllOnSubmit(okb.ProblemUsers.Where(x => x.ProblemId == problemId));
        okb.ProblemUsers.InsertAllOnSubmit(users.Select(x => new DataAccess.ProblemUser { ProblemId = problemId, UserId = x }));
        okb.SubmitChanges();
      }
    }
    public DataTransfer.BinaryData GetProblemData(long problemId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Problem>(x => x.BinaryData);
        okb.LoadOptions = dlo;
        return Convert.ToDto(okb.Problems.Where(x => x.Id == problemId).Select(x => x.BinaryData).FirstOrDefault());
      }
    }
    #endregion

    #region ProblemParameter Methods
    public DataTransfer.ProblemParameter GetProblemParameter(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.ProblemParameters.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.ProblemParameter> GetProblemParameters(long problemId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.ProblemParameters.Where(x => x.ProblemId == problemId).Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddProblemParameter(DataTransfer.ProblemParameter dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemParameter entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.ProblemParameters.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateProblemParameter(DataTransfer.ProblemParameter dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemParameter entity = okb.ProblemParameters.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteProblemParameter(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.ProblemParameter entity = okb.ProblemParameters.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.ProblemParameters.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region Result Methods
    public DataTransfer.Result GetResult(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return Convert.ToDto(okb.Results.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Result> GetResults(long algorithmId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        return okb.Results.Where(x => x.AlgorithmId == algorithmId).Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddResult(DataTransfer.Result dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Result entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.Results.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void UpdateResult(DataTransfer.Result dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Result entity = okb.Results.FirstOrDefault(x => x.Id == dto.Id);
        Convert.ToEntity(dto, entity);
        okb.SubmitChanges();
      }
    }
    public void DeleteResult(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Result entity = okb.Results.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Results.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region Experiment Methods
    public DataTransfer.Experiment GetExperiment(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterStringValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterStringValues);
        okb.LoadOptions = dlo;
        return Convert.ToDto(okb.Experiments.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Experiment> GetExperiments(long algorithmId, long problemId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterStringValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterStringValues);
        okb.LoadOptions = dlo;
        if ((algorithmId != 0) && (problemId != 0))
          return okb.Experiments.Where(x => (x.AlgorithmId == algorithmId) && (x.ProblemId == problemId)).Select(x => Convert.ToDto(x)).ToArray();
        else if (algorithmId != 0)
          return okb.Experiments.Where(x => x.AlgorithmId == algorithmId).Select(x => Convert.ToDto(x)).ToArray();
        else if (problemId != 0)
          return okb.Experiments.Where(x => x.ProblemId == problemId).Select(x => Convert.ToDto(x)).ToArray();
        else
          return okb.Experiments.Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddExperiment(DataTransfer.Experiment dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Experiment entity = Convert.ToEntity(dto); entity.Id = 0;

        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.AlgorithmParameterStringValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBlobValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterBoolValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterFloatValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterIntValues);
        dlo.LoadWith<Experiment>(x => x.ProblemParameterStringValues);
        okb.LoadOptions = dlo;

        var experiments = okb.Experiments.Where(x => ((x.AlgorithmId == entity.AlgorithmId) && (x.ProblemId == entity.ProblemId))).ToArray();
        ExperimentEqualityComparer comparer = new ExperimentEqualityComparer();
        Experiment exp = experiments.FirstOrDefault(x => comparer.Equals(x, entity));
        if (exp != null) {
          return exp.Id;
        } else {
          okb.Experiments.InsertOnSubmit(entity);
          okb.SubmitChanges();
          return entity.Id;
        }
      }
    }
    public void DeleteExperiment(long id) {
      foreach (DataTransfer.Run run in GetRuns(id))
        DeleteRun(run.Id);

      using (OKBDataContext okb = new OKBDataContext()) {
        IEnumerable<DataAccess.AlgorithmParameterBlobValue> algorithmParameterBlobValues = okb.AlgorithmParameterBlobValues.Where(x => x.ExperimentId == id);
        okb.AlgorithmParameterBlobValues.DeleteAllOnSubmit(algorithmParameterBlobValues);
        IEnumerable<DataAccess.AlgorithmParameterBoolValue> algorithmParameterBoolValues = okb.AlgorithmParameterBoolValues.Where(x => x.ExperimentId == id);
        okb.AlgorithmParameterBoolValues.DeleteAllOnSubmit(algorithmParameterBoolValues);
        IEnumerable<DataAccess.AlgorithmParameterFloatValue> algorithmParameterFloatValues = okb.AlgorithmParameterFloatValues.Where(x => x.ExperimentId == id);
        okb.AlgorithmParameterFloatValues.DeleteAllOnSubmit(algorithmParameterFloatValues);
        IEnumerable<DataAccess.AlgorithmParameterIntValue> algorithmParameterIntValues = okb.AlgorithmParameterIntValues.Where(x => x.ExperimentId == id);
        okb.AlgorithmParameterIntValues.DeleteAllOnSubmit(algorithmParameterIntValues);
        IEnumerable<DataAccess.AlgorithmParameterStringValue> algorithmParameterStringValues = okb.AlgorithmParameterStringValues.Where(x => x.ExperimentId == id);
        okb.AlgorithmParameterStringValues.DeleteAllOnSubmit(algorithmParameterStringValues);

        IEnumerable<DataAccess.ProblemParameterBlobValue> problemParameterBlobValues = okb.ProblemParameterBlobValues.Where(x => x.ExperimentId == id);
        okb.ProblemParameterBlobValues.DeleteAllOnSubmit(problemParameterBlobValues);
        IEnumerable<DataAccess.ProblemParameterBoolValue> problemParameterBoolValues = okb.ProblemParameterBoolValues.Where(x => x.ExperimentId == id);
        okb.ProblemParameterBoolValues.DeleteAllOnSubmit(problemParameterBoolValues);
        IEnumerable<DataAccess.ProblemParameterFloatValue> problemParameterFloatValues = okb.ProblemParameterFloatValues.Where(x => x.ExperimentId == id);
        okb.ProblemParameterFloatValues.DeleteAllOnSubmit(problemParameterFloatValues);
        IEnumerable<DataAccess.ProblemParameterIntValue> problemParameterIntValues = okb.ProblemParameterIntValues.Where(x => x.ExperimentId == id);
        okb.ProblemParameterIntValues.DeleteAllOnSubmit(problemParameterIntValues);
        IEnumerable<DataAccess.ProblemParameterStringValue> problemParameterStringValues = okb.ProblemParameterStringValues.Where(x => x.ExperimentId == id);
        okb.ProblemParameterStringValues.DeleteAllOnSubmit(problemParameterStringValues);

        DataAccess.Experiment entity = okb.Experiments.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Experiments.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion

    #region Run Methods
    public DataTransfer.Run GetRun(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Run>(x => x.ResultBlobValues);
        dlo.LoadWith<Run>(x => x.ResultBoolValues);
        dlo.LoadWith<Run>(x => x.ResultFloatValues);
        dlo.LoadWith<Run>(x => x.ResultIntValues);
        dlo.LoadWith<Run>(x => x.ResultStringValues);
        okb.LoadOptions = dlo;
        return Convert.ToDto(okb.Runs.FirstOrDefault(x => x.Id == id));
      }
    }
    public IEnumerable<DataTransfer.Run> GetRuns(long experimentId) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Run>(x => x.ResultBlobValues);
        dlo.LoadWith<Run>(x => x.ResultBoolValues);
        dlo.LoadWith<Run>(x => x.ResultFloatValues);
        dlo.LoadWith<Run>(x => x.ResultIntValues);
        dlo.LoadWith<Run>(x => x.ResultStringValues);
        okb.LoadOptions = dlo;
        return okb.Runs.Where(x => x.ExperimentId == experimentId).Select(x => Convert.ToDto(x)).ToArray();
      }
    }
    public long AddRun(DataTransfer.Run dto) {
      using (OKBDataContext okb = new OKBDataContext()) {
        DataAccess.Run entity = Convert.ToEntity(dto); entity.Id = 0;
        okb.Runs.InsertOnSubmit(entity);
        okb.SubmitChanges();
        return entity.Id;
      }
    }
    public void DeleteRun(long id) {
      using (OKBDataContext okb = new OKBDataContext()) {
        IEnumerable<DataAccess.ResultBlobValue> resultBlobValues = okb.ResultBlobValues.Where(x => x.RunId == id);
        okb.ResultBlobValues.DeleteAllOnSubmit(resultBlobValues);
        IEnumerable<DataAccess.ResultBoolValue> resultBoolValues = okb.ResultBoolValues.Where(x => x.RunId == id);
        okb.ResultBoolValues.DeleteAllOnSubmit(resultBoolValues);
        IEnumerable<DataAccess.ResultFloatValue> resultFloatValues = okb.ResultFloatValues.Where(x => x.RunId == id);
        okb.ResultFloatValues.DeleteAllOnSubmit(resultFloatValues);
        IEnumerable<DataAccess.ResultIntValue> resultIntValues = okb.ResultIntValues.Where(x => x.RunId == id);
        okb.ResultIntValues.DeleteAllOnSubmit(resultIntValues);
        IEnumerable<DataAccess.ResultStringValue> resultStringValues = okb.ResultStringValues.Where(x => x.RunId == id);
        okb.ResultStringValues.DeleteAllOnSubmit(resultStringValues);

        DataAccess.Run entity = okb.Runs.FirstOrDefault(x => x.Id == id);
        if (entity != null) okb.Runs.DeleteOnSubmit(entity);
        okb.SubmitChanges();
      }
    }
    #endregion
  }
}
